@@ -1,28 +1,30 @@
 #!/usr/bin/env python
 #
-#   Copyright information
+# The MIT License
 #
-#	Copyright (C) 2010-2018 Dilshod Temirkhodjaev <tdilshod@gmail.com>
+# Copyright (c) 2022 Dilshod Temirkhodjaev
 #
-#   License
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
 #
-#	This program is free software; you can redistribute it and/or modify
-#	it under the terms of the GNU General Public License as published by
-#	the Free Software Foundation; either version 2 of the License, or
-#	(at your option) any later version.
+# The above copyright notice and this permission notice shall be included in all
+# copies or substantial portions of the Software.
 #
-#	This program is distributed in the hope that it will be useful,
-#	but WITHOUT ANY WARRANTY; without even the implied warranty of
-#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-#	GNU General Public License for more details.
-#
-#	You should have received a copy of the GNU General Public License
-#	along with this program. If not, see <http://www.gnu.org/licenses/>.
-from __future__ import print_function
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
 
 __author__ = "Dilshod Temirkhodjaev <tdilshod@gmail.com>"
-__license__ = "GPL-2+"
-__version__ = "0.7.8"
+__license__ = "MIT"
+__version__ = "0.8"
 
 import csv, datetime, zipfile, string, sys, os, re, signal
 import xml.parsers.expat
@@ -147,6 +149,10 @@ class OutFileAlreadyExistsException(XlsxException):
     pass
 
 
+class XlsxValueError(XlsxException):
+    pass
+
+
 class Xlsx2csv:
     """
      Usage: Xlsx2csv("test.xslx", **params).convert("test.csv", sheetid=1)
@@ -154,6 +160,7 @@ class Xlsx2csv:
        xlsxfile - path to file or filehandle
      options:
        sheetid - sheet no to convert (0 for all sheets)
+       sheetname - sheet name to convert
        dateformat - override date/time format
        timeformat - override time format
        floatformat - override float format
@@ -218,8 +225,12 @@ class Xlsx2csv:
                 return s['index']
         return None
 
-    def convert(self, outfile, sheetid=1):
+    def convert(self, outfile, sheetid=1, sheetname=None):
         """outfile - path to file or filehandle"""
+        if sheetname:
+            sheetid = self.getSheetIdByName(sheetname)
+            if not sheetid:
+                raise XlsxException("Sheet '%s' not found" % sheetname)
         if sheetid > 0:
             self._convert(sheetid, outfile)
         else:
@@ -278,8 +289,7 @@ class Xlsx2csv:
             elif sys.version_info[0] == 3:
                 outfile = open(outfile, 'w+', encoding=self.options['outputencoding'], newline="")
             else:
-                sys.stderr.write("error: version of your python is not supported: " + str(sys.version_info) + "\n")
-                sys.exit(1)
+                raise XlsxException("error: version of your python is not supported: " + str(sys.version_info) + "\n")
             closefile = True
         try:
             writer = csv.writer(outfile, quoting=self.options['quoting'], delimiter=self.options['delimiter'],
@@ -287,8 +297,7 @@ class Xlsx2csv:
 
             sheets_filtered = list(filter(lambda s: s['index'] == sheet_index, self.workbook.sheets))
             if len(sheets_filtered) == 0:
-                eprint("Sheet with index %i not found or can't be handled" % sheet_index)
-                return 1
+                raise XlsxValueError("Sheet with index %i not found or can't be handled" % sheet_index)
 
             sheet_path = None
             # using sheet relation information
@@ -801,8 +810,7 @@ class Sheet:
 
                 # get format type
                 if not format_str:
-                    eprint("unknown format %s at %d" % (format_str, xfs_numfmt))
-                    return
+                    raise XlsxValueError("unknown format %s at %d" % (format_str, xfs_numfmt))
 
                 if format_str in FORMATS:
                     format_type = FORMATS[format_str]
@@ -863,9 +871,7 @@ class Sheet:
                         self.data = ("%f" % (float(self.data))).rstrip('0').rstrip('.')
 
                 except (ValueError, OverflowError):  # this catch must be removed, it's hiding potential problems
-                    eprint("Error: potential invalid date format.")
-                    # invalid date format
-                    pass
+                    raise XlsxValueError("Error: potential invalid date format.")
 
     def handleStartElement(self, name, attrs):
         has_namespace = name.find(":") > 0
@@ -884,7 +890,7 @@ class Sheet:
                     (name == 'v' or name == 'is') or (has_namespace and (name.endswith(':v') or name.endswith(':is')))):
             self.in_cell_value = True
             self.collected_string = ""
-        elif self.in_sheet and (name == 'row' or (has_namespace and name.endswith(':row'))) and ('r' in attrs):
+        elif self.in_sheet and (name == 'row' or (has_namespace and name.endswith(':row'))) and ('r' in attrs) and not ('hidden' in attrs and attrs['hidden'] == '1'):
             self.rowNum = attrs['r']
             self.in_row = True
             self.colIndex = 0
@@ -932,6 +938,7 @@ class Sheet:
                     d = self.mergeCells[self.mergeCells[self.colNum + self.rowNum]['copyFrom']]['value']
 
             self.columns[t - 1 + self.colIndex] = d
+            self.in_cell = False
 
         if self.in_row and (name == 'row' or (has_namespace and name.endswith(':row'))):
             if len(self.columns.keys()) > 0:
@@ -1017,17 +1024,21 @@ def convert_recursive(path, sheetid, outfile, kwargs):
             convert_recursive(fullpath, sheetid, outfile, kwargs)
         else:
             outfilepath = outfile
-            if len(outfilepath) == 0 and fullpath.lower().endswith(".xlsx"):
+            if isinstance(outfilepath, type(sys.stdout)):
+                outfilepath = fullpath[:-4] + 'csv'
+            elif os.path.isdir(outfilepath):
+                outfilepath = os.path.join(outfilepath, name[:-4] + 'csv')
+            elif len(outfilepath) == 0 and fullpath.lower().endswith(".xlsx"):
                 outfilepath = fullpath[:-4] + 'csv'
 
             print("Converting %s to %s" % (fullpath, outfilepath))
             try:
                 Xlsx2csv(fullpath, **kwargs).convert(outfilepath, sheetid)
             except zipfile.BadZipfile:
-                print("File %s is not a zip file" % fullpath)
+                raise InvalidXlsxFileException("File %s is not a zip file" % fullpath)
 
 
-if __name__ == "__main__":
+def main():
     try:
         signal.signal(signal.SIGPIPE, signal.SIG_DFL)
         signal.signal(signal.SIGINT, signal.SIG_DFL)
@@ -1103,8 +1114,7 @@ if __name__ == "__main__":
         (options, args) = parser.parse_args()
         if len(args) < 1:
             parser.print_usage()
-            sys.stderr.write("error: too few arguments" + os.linesep)
-            sys.exit(1)
+            sys.exit("error: too few arguments" + os.linesep)
         options.infile = args[0]
         options.outfile = len(args) > 1 and args[1] or None
 
@@ -1117,8 +1127,7 @@ if __name__ == "__main__":
     elif options.delimiter[0] == 'x':
         options.delimiter = chr(int(options.delimiter[1:]))
     else:
-        sys.stderr.write("error: invalid delimiter\n")
-        sys.exit(1)
+        sys.exit("error: invalid delimiter\n")
 
     if options.quoting == 'none':
         options.quoting = csv.QUOTE_NONE
@@ -1129,8 +1138,7 @@ if __name__ == "__main__":
     elif options.quoting == 'all':
         options.quoting = csv.QUOTE_ALL
     else:
-        sys.stderr.write("error: invalid quoting\n")
-        sys.exit(1)
+        sys.exit("error: invalid quoting\n")
 
     if options.lineterminator == '\n':
         pass
@@ -1141,8 +1149,7 @@ if __name__ == "__main__":
     elif options.lineterminator == '\\r\\n':
         options.lineterminator = '\r\n'
     else:
-        sys.stderr.write("error: invalid line terminator\n")
-        sys.exit(1)
+        sys.exit("error: invalid line terminator\n")
 
     if options.sheetdelimiter == '--------':
         pass
@@ -1153,8 +1160,7 @@ if __name__ == "__main__":
     elif options.sheetdelimiter[0] == 'x':
         options.sheetdelimiter = chr(int(options.sheetdelimiter[1:]))
     else:
-        sys.stderr.write("error: invalid sheet delimiter\n")
-        sys.exit(1)
+        sys.exit("error: invalid sheet delimiter\n")
 
     kwargs = {
         'delimiter': options.delimiter,
@@ -1190,9 +1196,12 @@ if __name__ == "__main__":
             if options.sheetname:
                 sheetid = xlsx2csv.getSheetIdByName(options.sheetname)
                 if not sheetid:
-                    raise XlsxException("Sheet '%s' not found" % options.sheetname)
+                    sys.exit("Sheet '%s' not found" % options.sheetname)
             xlsx2csv.convert(outfile, sheetid)
     except XlsxException:
         _, e, _ = sys.exc_info()
-        sys.stderr.write(str(e) + "\n")
-        sys.exit(1)
+        sys.exit(str(e) + "\n")
+
+
+if __name__ == "__main__":
+    main()
